require "memoist"

module ContainersCp
  module Mocks
    class S3Access
      include Import["logger", "gzip"]
      extend ::Memoist

      REGION = ENV['AWS_REGION'] || "us-east-1"

      def get_uncompressed(bucket, key, region: REGION, compression: :gz)
        content = get bucket, key, region: region
        case compression.to_sym
        when :gz
          gzip.uncompress content
        when :lz4
          ::LZ4.uncompress content
        else
          raise "Unsupported compression: #{compression}"
        end
      end

      def get(bucket, key, region: REGION)
        logger.debug "Download from #{bucket}:#{key}"
        s = get_stream(bucket, key, region: region)
        s.read
      rescue ::Errno::ENOENT
        raise ::Aws::S3::Errors::NoSuchKey.new("#{bucket}:#{key}", nil)
      ensure
        s&.close
      end

      def get_stream(bucket, key, region: REGION)
        file_name = build_path bucket, key
        ::File.new file_name, "rb"
      end

      private

      def build_path(bucket, key)
        path = ::File.join(folder(bucket), key)
        ensure_dir_exists path
        path
      end

      def ensure_dir_exists(path)
        f = ::File.dirname path
        ::FileUtils.mkdir_p f unless ::Dir.exist? f
      end

      def folder(bucket)
        ::File.join("/tmp", "cht-svc-containers-cp", "payload", bucket)
      end
    end
  end
end
